
package zero1hd.rhythmbullet.audio;

import com.badlogic.gdx.audio.Music.OnCompletionListener;
import com.badlogic.gdx.utils.Disposable;

public interface MusicManager extends Disposable {
	/**
	 * sets a integer variable to the current window of audio data the playback is at.
	 * Useful for efficiency because we compute once for that frame then get the values everytime it is required instead of calculating every time we get the index.
	 */
	public void playbackIndexUpdate();
	
	/**
	 * Gets the current position in seconds s
	 * @return the current frame of audio.
	 */
	public int getPlaybackIndexPosition();
	
	/**
	 * returns the read window size.
	 * @return
	 */
	public int getReadWindowSize();
	
	/**
	 * read in samples and fills the array.
	 * @param samples the array that should contain said samples
	 * @return amount read. Will return 0 if end of stream.
	 */
	public int readSamples(float[] samples);
	
	/**
	 * returns sample count.
	 * Can be inaccurate.
	 * @return
	 */
	public long getSampleCount();
	
	/**
	 * returns duration of song in seconds
	 * @return
	 */
	public float getDuration();
	
	/**
	 * returns sample rate
	 * @return sample rate as float
	 */
	public float getSampleRate();
	
	/**
	 * The current numerical value of the last window read
	 * @return
	 */
	public int getCurrentReadWindowIndex();

	public void pause();

	public void play();

	public boolean isPlaying();

	public float getPosition();

	public void setPosition(float position);

	public void setOnCompletionListener(OnCompletionListener listener);

	public void setVolume(float percent);
	
	/**
	 * If the the properties of the song are done loading
	 * @return whether its done loading
	 */
	public boolean isFinishedLoading();
}
