package zero1hd.rhythmbullet.graphics.ui.pages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.SongListController;
import zero1hd.rhythmbullet.events.OnDifferentSongListener;
import zero1hd.rhythmbullet.graphics.ui.components.TitleBarVisualizer;

public class MainPage extends Page implements OnDifferentSongListener {
	private Label versionLabel;
	
	private SongListController sc;
	private TitleBarVisualizer titleBar;
	private Table table;
	private TextButton playButton;
	private TextButton optionsButton;
	private TextButton quitButton;
	public MainPage(RhythmBullet core, Vector3 targetPosition, SongListController sc) {
		this.sc = sc;
		
		titleBar = new TitleBarVisualizer(core.getAssetManager());
		addActor(titleBar);
		
		titleBar.getHvisual().setMM(sc.getCurrentSong());
		
		sc.addOnDifferentSongListener(this);
		
		versionLabel = new Label("Version: " + RhythmBullet.VERSION, core.getDefaultSkin(), "sub-font",
				core.getDefaultSkin().getColor("default"));
		versionLabel.setPosition(3, 3);
		addActor(versionLabel);
		
		table = new Table();
		table.setSize(getWidth(), titleBar.getY());
		table.align(Align.center);
		table.defaults().space(10f);
		addActor(table);
		
		playButton = new TextButton("Start!", core.getDefaultSkin());
		playButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				//TODO move to play screen
			}
		});
		table.add(playButton).width(Gdx.graphics.getWidth()*0.2f);
		
		table.row();
		
		optionsButton = new TextButton("Options", core.getDefaultSkin(), "sub");
		optionsButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				targetPosition.x = Gdx.graphics.getWidth()*1.5f;
			}
		});
		table.add(optionsButton).fillX();
		
		table.row();
		
		quitButton = new TextButton("Quit", core.getDefaultSkin(), "sub");
		quitButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				Gdx.app.exit();
			}
		});
		table.add(quitButton).fillX();
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
	}

	@Override
	public void onDifferentSong(MusicManager mdp) {
		titleBar.getHvisual().setMM(mdp);
	}
	
	@Override
	public void dispose() {
		super.dispose();
	}
}
