package zero1hd.rhythmbullet.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.SongList;
import zero1hd.rhythmbullet.audio.SongListController;
import zero1hd.rhythmbullet.graphics.ui.pages.CreditsPage;
import zero1hd.rhythmbullet.graphics.ui.pages.KeybindOptionsPage;
import zero1hd.rhythmbullet.graphics.ui.pages.MainPage;
import zero1hd.rhythmbullet.graphics.ui.pages.OptionsPage;
import zero1hd.rhythmbullet.graphics.ui.pages.VideoOptionsPage;
import zero1hd.rhythmbullet.util.TransitionAdapter;

public class MainMenu extends ScreenAdapter implements TransitionAdapter {
	public Stage stage;
	private Vector3 targetPosition;
	
	private MainPage mainPage;
	private OptionsPage optionsPage;
	private CreditsPage creditsPage;
	private KeybindOptionsPage keybindPage;
	private VideoOptionsPage graphicsPage;
	private RhythmBullet core;
	
	private SongListController sc;
	private float lerpAlpha;
	
	private ShaderProgram gaussianBlurShader;
	private ShaderProgram brightFilterShader;
	private ShaderProgram combineShader;
	private FrameBuffer lightFilterBuffer;
	private FrameBuffer normalBuffer;
	private FrameBuffer hBlur, vBlur;
	private TextureRegion fboRegion;
	private int fboSize;
	private int blurlvl = 2;
	private Batch batch;
	private ScreenViewport screenViewport;
	
	public MainMenu(final RhythmBullet core) {
		this.core = core;
		stage = new Stage(new ScreenViewport());
		targetPosition = new Vector3(stage.getCamera().position);
		
		SongList songList = new SongList();
		songList.setSearchPath(core.getPrefs().getString("music dir"));
		songList.refresh();
		
		sc = new SongListController(songList, core.getPrefs());
		sc.setAutoPlay(true);
		sc.setShuffle(true);
		
		postTransition();
	}
	
	@Override
	public void postTransition() {
		loadShaders();
		
		mainPage = new MainPage(core, targetPosition, sc);
		mainPage.setPosition(0, 0);
		stage.addActor(mainPage);
		//End main menu
		
		keybindPage = new KeybindOptionsPage(core.getDefaultSkin(), core.getAssetManager());
		keybindPage.setPosition(1f*Gdx.graphics.getWidth(), -1f*Gdx.graphics.getHeight());
		stage.addActor(keybindPage);
		
		graphicsPage = new VideoOptionsPage(core.getDefaultSkin(), core.getPrefs());
		
		graphicsPage.setPosition(1f*Gdx.graphics.getWidth(), 1f*Gdx.graphics.getHeight());
		stage.addActor(graphicsPage);
		
		optionsPage = new OptionsPage(core, targetPosition, keybindPage, sc);
		optionsPage.setPosition(Gdx.graphics.getWidth(), 0);
		stage.addActor(optionsPage);
		
		creditsPage = new CreditsPage(core.getDefaultSkin());
		creditsPage.setPosition(0, Gdx.graphics.getHeight());
		stage.addActor(creditsPage);
		
		
		stage.addListener(new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.ESCAPE) {
					stage.unfocusAll();
					if (targetPosition.x != 0.5f*Gdx.graphics.getWidth() || targetPosition.y != 0.5f*Gdx.graphics.getHeight()) {
						targetPosition.x = 0.5f*Gdx.graphics.getWidth();
						targetPosition.y = 0.5f*Gdx.graphics.getHeight();
					}
					keybindPage.unselect();
				}
				return super.keyUp(event, keycode);
			}
		});	
		
		stage.addListener(new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				if (stage.hit(x, y, true) == null) {
					stage.unfocusAll();
					keybindPage.unselect();
				}
				super.clicked(event, x, y);
			}
		});
		
		Gdx.app.debug("Post Transition", "Beginning screen setup for Main menu.");
	}
	public void loadShaders() {
		if (core.getPrefs().getBoolean("glow shader")) {
			batch = new SpriteBatch();
			screenViewport = new ScreenViewport();
			screenViewport.update(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			((OrthographicCamera) screenViewport.getCamera()).setToOrtho(false, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			
			Gdx.app.debug("Shader", "using glow shader");
			brightFilterShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/bright_filter.fsh"));
			if (!brightFilterShader.isCompiled()) {
				System.err.println(brightFilterShader.getLog());
				System.exit(0);
			}
			if (brightFilterShader.getLog().length() != 0) {
				System.out.println(brightFilterShader.getLog());
			}
				
			gaussianBlurShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/gaussian_blur.fsh"));
			if (!gaussianBlurShader.isCompiled()) {
				System.err.println(gaussianBlurShader.getLog());
				System.exit(0);
			}
			if (gaussianBlurShader.getLog().length() != 0) {
				System.out.println(gaussianBlurShader.getLog());
			}
			
			combineShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/combine.fsh"));
			if (!combineShader.isCompiled()) {
				System.err.println(combineShader.getLog());
				System.exit(0);
			}
			if (combineShader.getLog().length() != 0) {
				System.out.println(combineShader.getLog());
			}
			
			
			if (Gdx.graphics.getWidth() < 1024) {
				fboSize = 2048;
			} else if (Gdx.graphics.getWidth() < 2048) {
				fboSize = 4096;
			} else {
				fboSize = 4096;
			}
			
			lightFilterBuffer = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			normalBuffer = new FrameBuffer(Format.RGBA8888, fboSize, fboSize, false);
			hBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			vBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);

			fboRegion = new TextureRegion(normalBuffer.getColorBufferTexture());
			fboRegion.flip(false, true);

			combineShader.begin();
			combineShader.setUniformi("u_texture1", 1);
			combineShader.end();
			
			vBlur.getColorBufferTexture().bind(1);
			
			Gdx.gl.glActiveTexture(GL20.GL_TEXTURE0);
		}
		
		ShaderProgram.pedantic = false;
	}
	
	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(0.1f, 0.1f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		stage.act(delta);
		blurlvl = 5;
		if (gaussianBlurShader != null) {
			//Begin drawing a normal version of screen
			stage.getViewport().apply();
			normalBuffer.begin();
			Gdx.gl.glClearColor(0.22f, 0f, 0f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			stage.draw();
			normalBuffer.end();

			//BEGINNING NORMAL SCREEN RENDER
			screenViewport.apply();
			
//			Begin light filtering
			lightFilterBuffer.begin();
			Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			fboRegion.setTexture(normalBuffer.getColorBufferTexture());
			batch.setShader(brightFilterShader);
			batch.setProjectionMatrix(screenViewport.getCamera().combined);
			batch.begin(); //BATCH STARTS HERE
			batch.draw(fboRegion, 0, 0, stage.getWidth(), stage.getHeight());
			batch.flush();
			lightFilterBuffer.end();
//			
			for (int i = 0; i < blurlvl; i++) {
//				Horizontal gaussian blur
				hBlur.begin();
				if (i > 0) {
					fboRegion.setTexture(vBlur.getColorBufferTexture());
				} else {
					fboRegion.setTexture(lightFilterBuffer.getColorBufferTexture());
				}
				batch.setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 1);
				batch.draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				batch.flush();
				hBlur.end();
				
//				//Vertical gaussian blur
				vBlur.begin();
				fboRegion.setTexture(hBlur.getColorBufferTexture());
				batch.setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 0);
				batch.draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				batch.flush();
				vBlur.end();
			}

			Gdx.gl.glClearColor(0f, 0f, 0f, 0f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			batch.setShader(combineShader);
			fboRegion.setTexture(normalBuffer.getColorBufferTexture());
			batch.draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
			batch.setShader(null);
			batch.end(); //STAGE BATCH ENDS HERE

		} else {
			stage.draw();
		}
        
		if (stage.getCamera().position.x != targetPosition.x || stage.getCamera().position.y != targetPosition.y) {
			stage.getCamera().position.lerp(targetPosition, lerpAlpha);
			stage.getViewport().apply();
		}
		
		super.render(delta);
	}
	@Override
	public void preTransition() {
		stage.clear();
		mainPage.dispose();
		optionsPage.dispose();
		creditsPage.dispose();
		keybindPage.dispose();
	}
	
	@Override
	public void show() {
		Gdx.input.setInputProcessor(stage);
		sc.play();
		calcLerpAlpha(Gdx.graphics.getWidth());
		super.show();
	}
	
	@Override
	public void hide() {
		saveAll();
		super.hide();
	}
	
	public void saveAll() {
		if (optionsPage != null) {
			optionsPage.saveOptions(core.getPrefs());
			core.getPrefs().flush();
		}
	}
	
	@Override
	public void resize(int width, int height) {
		stage.getViewport().update(width, height, false);
		targetPosition.x = width/2;
		targetPosition.y = height/2;
		calcLerpAlpha(width);
		super.resize(width, height);
	}
	
	@Override
	public void dispose() {
		stage.dispose();
		super.dispose();
	}
	
	private void calcLerpAlpha(int width) {
		if (width <= 3835) {
			lerpAlpha = 0.15f;
		} else {
			lerpAlpha = 0.15f;
		}
	}
}
